// C++ includes
#include <iostream>
#include <memory>
#include <string>

// WPILIB includes
#include <IterativeRobot.h>
#include <LiveWindow/LiveWindow.h>
#include <SmartDashboard/SendableChooser.h>
#include <SmartDashboard/SmartDashboard.h>


//Team302 includes
//#include <subsys/interfaces/IChassis.h>
#include <teleopControl/TeleopControl.h>
//#include <teleopDrive/TeleopDrive.h>
//#include <subsys/factories/ChassisFactory.h>
#include <test/ITest.h>
#include <test/Prototype.h>


class Robot: public frc::IterativeRobot
{
	public:
		//===========================================================================================
		// Method:			RobotInit
		// Description:		Initializes the robot when it it turned on (it is only run once).
		//                  This is where class attributes are created, so that they can be used
		// 					by all of the methods in the class.
		//===========================================================================================
		void RobotInit() override
		{
			printf( "RobotInit \n" );		// Debugging

			// Display the autonomous choices on the dashboard for selection.
			m_autonChooser.AddDefault(m_autoNameDefault, m_autoNameDefault);
			m_autonChooser.AddObject(m_autoNameCustom, m_autoNameCustom);
			frc::SmartDashboard::PutData("Auto Modes", &m_autonChooser);

			// Teleop Drive
//			m_teleopDrive = new TeleopDrive();

			// Prototyping
			m_proto = new ProtoType();
		}

		//===========================================================================================
		// Method:		DisabledInit
		// Description:	Called whenever the robot gets disabled (once when it gets disabled).
		//===========================================================================================
		void DisabledInit() override
		{
			printf( "DisabledInit \n" );		// Debugging
			m_proto->Stop();
		}

		//===========================================================================================
		// Method:		DisabledPeriodic
		// Description:	Called every 20 milliseconds when the robot is disabled.
		//===========================================================================================
	    void DisabledPeriodic() override
	   	{
		//	printf( "DisabledPeriodic \n" );
	   	}

		//===========================================================================================
	    // Method:		AutonomousInit
	    // Description: Initialized autonomous (called once when autonomous starts).  It uses the
		// 				chooser created in RobotInit to select the desires autonomous routine.
		//===========================================================================================
		void AutonomousInit() override
		{
			printf( "AutonInit \n" );		// debugging

			// Get the selected Auton Option
			m_autoSelected = m_autonChooser.GetSelected();			// Java/C++ SmartDashboard
			// std::string autoSelected = SmartDashboard::GetString("Auto Selector", autoNameDefault);	// Labview DashBoard
			std::cout << "Auto selected: " << m_autoSelected << std::endl;

			// Process the Auton selection
			if ( m_autoSelected == m_autoNameCustom )
			{
				// Custom Auto goes here
			} else
			{
				// Default Auto goes here
			}
		}

		//===========================================================================================
		// Method:  		AutonomousPeriodic
		// Description:		Run the selected autonomous routine (every 20 milliseconds)
		//===========================================================================================
		void AutonomousPeriodic() override
		{
			printf( "AutonPeriodic \n" );

			if (m_autoSelected == m_autoNameCustom)
			{
				// Custom Auto goes here
			}
			else
			{
				// Default Auto goes here
			}
		}


		//===========================================================================================
		// Method:  		TeleopInit
		// Description:		Initializes teleop processing when teleop starts.
		//===========================================================================================
		void TeleopInit() override
		{
			printf( "TeleopInit \n" );		// debugging
		}


		//===========================================================================================
		// Method:  		TeleopPeriodic
		// Description:		Run robot in teleop mode (every 20 milliseconds)
		//===========================================================================================
		void TeleopPeriodic() override
		{
			printf( "TeleopPeriodic \n" );		// debugging

//			m_teleopDrive->Drive();
//			ChassisFactory::GetInstance()->GetIChassis()->PrintDebug();

		}

		//===========================================================================================
		// Method:  		TestInit
		// Description:		Initializes the test environment when test starts
		//===========================================================================================
	    void TestInit() override
	   	{
			printf( "TestInit \n" );		// debugging
			m_proto->Init( ITest::PROTOTYPE_TEST );
	   	}


		//===========================================================================================
		// Method:  		TestPeriodic
		// Description:		Run the test routine (every 20 milliseconds)
		//===========================================================================================
		void TestPeriodic() override
		{
//			printf( "TestPeriodic \n" );	// debugging

			lw->Run();

			m_proto->Periodic();
		}

	//================================================================================================
	private:
		frc::LiveWindow* lw = LiveWindow::GetInstance();						// used in test mode

		frc::SendableChooser<std::string> 	m_autonChooser;						// Auton mode selector
		const std::string 					m_autoNameDefault = "Default";		// Default auton option
		const std::string 					m_autoNameCustom = "My Auto";		// Custom auton option
		std::string 						m_autoSelected;						// Selected auton mode

		ProtoType* 							m_proto;

//		TeleopDrive* 						m_teleopDrive;

};

START_ROBOT_CLASS(Robot)
