/*
 * ManipulatorDefn.cpp
 */

#include <subsys/xmlParsing/MotorDefn.h>

#include <subsys/components/DragonTalon.h>

#include <subsys/interfaces/IChassis.h>
#include <subsys/xmlParsing/ManipulatorDefn.h>
#include <subsys/factories/ManipulatorFactory.h>
#include <subsys/ProtoSubsys.h>

#include <iostream>

#include <SmartDashboard/SmartDashboard.h>

#include <thirdParty/puxixml/pugixml.hpp>


//-----------------------------------------------------------------------
// Method:      ParseXML
// Description: Parse a Chassis XML element and create an IChassis from
//              its definition.
// Returns:     IChassis*        	chassis (or nullptr if XML
//                                  is ill-formed)
//-----------------------------------------------------------------------
IChassis* ManipulatorDefn::ParseXML
(
	pugi::xml_node      node
)
{
    IChassis* chassis = nullptr;
    ManipulatorFactory::MANIPULATOR_TYPE type = ManipulatorFactory::UNKNOWN_MANIPULATOR;

    bool hasError 		= false;

    // process attributes
    for (pugi::xml_attribute attr = node.first_attribute(); attr; attr = attr.next_attribute())
    {
    	printf( "Manipulator attr: %s = %s \n ", attr.name(), attr.value() );

        if ( strcmp( attr.name(), "type" ) == 0 )
        {
        	if ( strcmp( attr.value(), "prototype" ) == 0 )
        	{
        		type = ManipulatorFactory::PROTOTYPE_MANIPULATOR;
        	}
        	else if ( strcmp( attr.value(), "climber") == 0 )
        	{
        		type = ManipulatorFactory::CLIMBER_MANIPULATOR;
        	}
        	else
        	{
        		printf( "unknown manipulator type %s \n ", attr.value() );
        		hasError = true;
        	}
        }
        else
        {
            std::cout << " invalid attribute " << attr.name();
            hasError = true;
        }
    }

    std::vector<DragonTalon*> talons;
    for (pugi::xml_node child = node.first_child(); child; child = child.next_sibling())
    {
    	printf( "Child %s \n ", child.name() );
    	if ( strcmp( child.name(), "motor") == 0 )
    	{
    		talons.emplace_back( MotorDefn::ParseXML( child ) );
    	}
    	else if ( strcmp( child.name(), "analogSensor") == 0 )
    	{

    	}
    	else if ( strcmp( child.name(), "digitalSensor") == 0 )
    	{

    	}
    	else if ( strcmp( child.name(), "solenoid") == 0 )
    	{

    	}
    	else if ( strcmp( child.name(), "pcm") == 0 )
    	{

    	}
    }


    // create instance
    if ( !hasError )
    {
        printf( "Creating Manipulator \n");
        ProtoSubsys* proto = nullptr;
        switch ( type )
        {
        	case ManipulatorFactory::PROTOTYPE_MANIPULATOR:
        		proto = ManipulatorFactory::GetPrototype();
        		proto->AddTalons( talons );
        		break;

        	case ManipulatorFactory::CLIMBER_MANIPULATOR:
        		break;

        	default:
        		printf( "invalid manipulator \n ");
        		break;
        }
    }
    return chassis;
}
