/*
 * RobotDefn.cpp
 *
 */

#include <subsys/xmlParsing/RobotDefn.h>
#include <subsys/xmlParsing/ChassisDefn.h>
#include <subsys/xmlParsing/ManipulatorDefn.h>

#include <iostream>

#include <SmartDashboard/SmartDashboard.h>

#include <thirdParty/puxixml/pugixml.hpp>

#include <subsys/xmlParsing/MotorDefn.h>
#include <subsys/components/DragonTalon.h>
#include <subsys/xmlParsing/ManipulatorDefn.h>

//-----------------------------------------------------------------------
// Method:      ParseXML
// Description: Parse a robot.xml file
// Returns:     void
//-----------------------------------------------------------------------
void RobotDefn::ParseXML()
{
	const char* filename = "/home/lvuser/config/robot.xml";

    pugi::xml_document doc;
    pugi::xml_parse_result result = doc.load_file( filename );


    if ( result )
    {
        printf( "XML file [ %s ] parsed without errors \n ", filename );

        pugi::xml_node parent = doc.root();
        for (pugi::xml_node node = parent.first_child(); node; node = node.next_sibling())
        {
        	printf( "Node: %s \n ", node.name() );

            for (pugi::xml_node child = node.first_child(); child; child = child.next_sibling())
            {
            	printf( "Child %s \n ", child.name() );
            	if ( strcmp( child.name(), "chassis") == 0 )
            	{
            		ChassisDefn::ParseXML( child );
            	}
            	else if ( strcmp( child.name(), "manipulator") == 0 )
            	{
            		ManipulatorDefn::ParseXML( child );
            	}
            }
        }
    }
    else
    {
        std::cout << "XML [" << filename << "] parsed with errors, attr value: [" << doc.child("prototype").attribute("attr").value() << "]\n";
        std::cout << "Error description: " << result.description() << "\n";
        std::cout << "Error offset: " << result.offset << " (error at [..." << (filename + result.offset) << "]\n\n";
    }
}
