/*
 * AnalogInputDefn.cpp
 *
 */

#include <AnalogInput.h>
#include <subsys/xmlParsing/AnalogInputDefn.h>
#include <SmartDashboard/SmartDashboard.h>

#include <thirdParty/puxixml/pugixml.hpp>


using namespace frc;

//-----------------------------------------------------------------------
// Method:      ParseXML
// Description: Parse a motor XML element and create a AnalogInput from
//              its definition.
// Returns:     AnalogInput*        AnalogInput (or nullptr if XML
//                                  is ill-formed
//-----------------------------------------------------------------------
AnalogInput* AnalogInputDefn::ParseXML
(
    pugi::xml_node      motorNode
)
{
	AnalogInput* sensor = nullptr;

	enum SENSOR_TYPE
	{
		GENERAL,
		GYRO,
		POTENTIOMETER
	};

	enum SENSOR_USAGE
	{
		PRESSURE_GAUGE
	};
	SENSOR_TYPE					type = GENERAL;
	SENSOR_USAGE				usage = PRESSURE_GAUGE;
	int 						analogID = 0;
    float						voltageMin = 0.0;
    float						voltageMax = 5.0;
    float 						outputMin  = 0.0;
    float						outputMax  = 1.0;

    bool hasError = false;

    for (pugi::xml_attribute attr = motorNode.first_attribute(); attr; attr = attr.next_attribute())
    {
        if ( strcmp( attr.name(), "type" ) == 0 )
        {
            type = attr.as_int();
        }
        else if ( strcmp( attr.name(), "usage" ) == 0 )
        {
            usage = attr.as_int();
        }
        else if ( strcmp( attr.name(), "analogId" ) == 0 )
        {
        	analogID = attr.as_int();
        }
        else if ( strcmp( attr.name(), "voltageMin" ) == 0 )
        {
        	voltageMin = attr.as_float();
        }
        else if ( strcmp( attr.name(), "voltageMax" ) == 0 )
        {
        	voltageMax = attr.as_float();
        }
        else if ( strcmp( attr.name(), "outputMin" ) == 0 )
        {
        	outputMin = attr.as_float();
        }
        else if ( strcmp( attr.name(), "outputMax" ) == 0 )
        {
        	outputMax = attr.as_float();
        }
        else
        {
            printf( " invalid attribute &s \n", attr.name() );
            hasError = true;
        }
    }

    if ( !hasError )
    {
        printf( "Creating AnalogInput \n");

    }
    return sensor;
}
