/*
 * EncoderDefn.h
 *
 *  Created on: Aug 18, 2017
 *      Author: witcpale
 */

#ifndef SRC_INCLUDES_DEFINITIONS_SOLENOIDDEFN_H_
#define SRC_INCLUDES_DEFINITIONS_SOLENOIDDEFN_H_

namespace Team302
{

	class EncoderDefn
	{
		public:
			enum EncoderSampleRate
			{
				RATE_1X,
				RATE_2X,
				RATE_4X
			};
			enum DistanceUnit
			{
				INCHES,
				MM
			};
			EncoderDefn() = delete;
			EncoderDefn
			(
				int					channelA,				// <I> - first the counting channels or -1 if plugged directly into motor controller
				int					channelB,				// <I> - second the counting channels or -1 if plugged directly into motor controller
				bool 				isReverse,				// <I> - true, it will reverse the sensor reading
				EncoderSampleRate 	sampleRate,				// <I> - sampling rate from the specifications
				int		 			numChannels,    		// <I> - number of channels
				int		 			pulsesPerRev,			// <I> - pulses per revolution per the specifications
				float 				encodeRevsPerOutputRev,	// <I> - This is the gear ratio if the encoder is on the motor not the output shaft
				float	 			outputDiameter,			// <I> - Often wheel diameter.  This is used to calculate distance, speed, etc.
				DistanceUnit	 	unit					// <I> - units of the OutputDiameter
			);
			virtual ~EncoderDefn() = default;

			inline int GetChannelA() { return m_channelA; }
			inline int GetChannelB() { return m_channelB; }
			inline bool IsReversed() { return m_reversed; }
			inline EncoderSampleRate GetSampleRate() { return m_sampleRate; }
			inline int GetNumChannels() { return m_numChannels; }
			inline int GetPulsesPerRev() { return m_pulsesPerRev; }
			inline float GetOutputRatio() { return m_outputRatio; }
			inline float GetOutputDiamer() { return m_outputDiameter; }
			inline DistanceUnit GetDistanceUnit() { return m_unit; }

		private:
			int 				m_channelA;
			int 				m_channelB;
			bool				m_reversed;
			EncoderSampleRate	m_sampleRate;
			int 				m_numChannels;
			int 				m_pulsesPerRev;
			float				m_outputRatio;
			float				m_outputDiameter;
			DistanceUnit		m_unit;
	};

} /* namespace Team302 */

#endif /* SRC_INCLUDES_DEFINITIONS_SOLENOIDDEFN_H_ */
